/*
 * file ResourceList.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 * 
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import java.util.List;
import java.util.NoSuchElementException;

import javax.wvcm.WvcmException.ReasonCode;


/**
 * A list containing {@link Resource} objects,
 * used to perform operations on all elements of the list.
 * A new ResourceList is created with {@link Provider#resourceList}
 * 
 * @since 1.0
 */
public interface ResourceList<T extends Resource> extends List<T> {

    /**
     * An iterator of the results of applying a method to
     * each element of a ResourceList.
     * @see java.util.Iterator
     */
    public interface ResponseIterator <V> {
        /**
         * Test whether the iteration has more elements.
         * 
         * @return true if the iteration has more elements.
         * @see java.util.Iterator#hasNext
         */
        public boolean hasNext();

        /**
         * Get the result of applying the method to the next element of the ResourceList.
         * 
         * @return if the method succeeded on the corresponding element of the ResourceList,
         * the object returned by that method is returned; otherwise,
         * a WvcmException is thrown. For methods that return void, next() returns null.
         * @throws WvcmException indicates that the method failed on this element
         * of the ResourceList.
         * If {@link WvcmException.ReasonCode} is {@link ReasonCode#ABORTED},
         * it does not indicate the method has failed on this element, 
         * but instead indicates that an abort requested through {@link Feedback}
         * has terminated the ResponseIterator.
         * After this exception is handled, unless the reason code is {@link ReasonCode#ABORTED},
         * further calls to hasNext() and next() are valid,
         * and will process the remaining elements of the ResourceList.
         * @throws NoSuchElementException iteration has no more elements.
         * @see java.util.Iterator#next
         */
        public V next() throws WvcmException, NoSuchElementException;

        /**
         * Release any resources associated with this ResponseIterator.
         * This should be called if the client is done with this ResponseIterator
         * even though the last call to hasNext has returned true.
         */
        public void release();
    };

    /**
     * Apply {@link Resource#doReadProperties}
     * to each element of this ResourceList.
     * 
     * @param feedback the properties available in the returned proxies.
     * @return an iterator over the results of the method on each element in this ResourceList.
     */
    public <V> ResponseIterator<V> doReadProperties(Feedback feedback) throws WvcmException;

    /**
     * Apply {@link Resource#doReadProperties}
     * to each element of this ResourceList.
     * 
     * @param feedback the properties available in the returned proxies.
     * @param context additional context for the request (commonly a {@link Workspace} or {@link Stream}>
     * @return an iterator over the results of the method on each element in this ResourceList.
     */
    public <V> ResponseIterator<V> doReadContextProperties(Resource context, Feedback feedback) throws WvcmException;

    /**
     * Apply {@link Resource#doWriteProperties doWriteProperties}
     * to each element of this ResourceList.
     * 
     * @param feedback the properties available in the returned proxies.
     * @return an iterator over the results of the method on each element in this ResourceList.
     */
    public <V> ResponseIterator<V> doWriteProperties(Feedback feedback) throws WvcmException;

}
